package com.book.server.aspect;

import com.book.server.annotation.CacheFind;
import com.book.server.common.util.CacheUtil;
import com.book.server.utils.JsonUtils;
import com.book.server.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.concurrent.TimeUnit;

@Component
@Aspect
@Slf4j
public class CacheAOP {

    @Autowired
    private RedisUtil redisUtil;

    /**
     * AOP缓存实现的业务策略
     * 1：切入点表达式应该拦截 @CacheFind注解
     * 2：通知方法： 环绕通知
     * 注意：
     * 如果使用环绕通知，这必须在第一个参数的位置添加ProceedingJoinPoint
     * 完成redis配置  key
     * <p>
     * 动态获取注解参数的步骤
     * 1.@annotation(cacheFind)   切入点表达式要求拦截一个类型为cacheFind注解。
     * 2.并且利用连接点为参数中的cacheFind赋值
     */
    @Around("@annotation(cacheFind)")  //输入的是切入点的表达式
    public Object around(ProceedingJoinPoint joinPoint, CacheFind cacheFind) {
        Object result = null;
        try {
            String key = CacheUtil.getKeyWithPre(cacheFind.key(), joinPoint.getArgs());

            if (cacheFind.delete()) {
                // 删除缓存
                redisUtil.remove(key);
                result = joinPoint.proceed(); //执行目标方法
            } else if (cacheFind.update()) {
                // 更新缓存
                result = joinPoint.proceed();
                setResult(key, result, cacheFind);
            } else {
                // 默认，查询缓存，不存在则set
                Object o = redisUtil.get(key);
                MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
                Class returnClass = methodSignature.getReturnType();
                Type type = methodSignature.getMethod().getGenericReturnType();
                //3.校验redis中是否有数据，
                if (o != null) {
                    if (type instanceof ParameterizedType) {
                        //如果是数组
                        ParameterizedType pt = (ParameterizedType) type;
                        Type modelType = pt.getActualTypeArguments()[0];
                        result = JsonUtils.getList(o.toString(), Class.forName(modelType.getTypeName()));
                    } else {
                        //如果是普通对象
                        result = JsonUtils.getObject(o.toString(), returnClass);
                    }

                } else {
                    //没缓存，查询数据库
                    result = joinPoint.proceed();
                    //将数据保存到redis
                    setResult(key, result, cacheFind);
                }
            }
            return result;

        } catch (Throwable throwable) {
            throwable.printStackTrace();
            throw new RuntimeException(throwable);
        }
    }

    /**
     * 根据缓存时间，缓存数据
     *
     * @param key
     * @param result
     * @param cacheFind
     */
    private void setResult(String key, Object result, CacheFind cacheFind) {
        if (cacheFind.seconds() == 0) {
            redisUtil.setWithTime(key, JsonUtils.toJsonStr(result));
        } else {
            redisUtil.set(key, JsonUtils.toJsonStr(result), cacheFind.seconds(), TimeUnit.SECONDS);
        }
    }
}
