package com.book.server.controller;

import com.book.dao.cps.mapper.BookMapper;
import com.book.dao.cps.pojo.Book;
import com.book.dao.cps.pojo.SearchKeyword;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.server.common.entity.Result;
import com.book.server.common.entity.ResultCode;
import com.book.dao.VO.*;
import com.book.server.service.BlockCacheGuava;
import com.book.server.service.BookService;
import com.book.server.service.CacheService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureOrder;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * 书籍
 */
@RestController
@RequestMapping("/api/book")
@Slf4j
public class BookController extends BaseController {
    @Autowired
    private BookService bookService;


    /**
     * 书库查询
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/query")
    public Result<List<BookRes>> query(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.query(queryVO));
    }

    /**
     * 书城分页推荐-精彩推荐
     * novel_end_recommend
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/recommend")
    public Result<List<BookRes>> recommendByRead(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.recommendByRead(queryVO));
    }

    /**
     * 主编推荐
     * smart_recommand
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/smartRecommand")
    public Result<List<BookRes>> smartRecommand(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.smartRecommand(queryVO));
    }

    /**
     * 热门推荐4个
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/topRecommend")
    public Result<List<BookRes>> topRecommend(@RequestBody QueryVO queryVO) {
        List<BookRes> books = bookService.topRecommend(queryVO);
        return Result.success(books);
    }

    /**
     * 书城block
     *
     * @param pageId
     * @return
     */
    @GetMapping("/getBlockByPageId")
    //@Cacheable(value = "block", key = "#pageId")
    public Result<List<BlockRes>> getBlockByPageId(Integer pageId) throws ExecutionException {
        List<BlockRes> list = bookService.getBlockByPageId(pageId);
        return Result.byObject(list);
    }


    /**
     * 导入书库
     * 注意：此代码仅限拉取书籍使用，只能运行一次。
     *
     * @return
     */

    @GetMapping("/insertBooks-F527BC16-2BEB-4F6A-8D56-121125F9D2AE")
    public Result insertBooks() {
        //boolean open = false;
        //if (open) {
        bookService.insertBooks();
        //}
        return Result.byObject("OK");
    }

    /**
     * 获取章节列表
     *
     * @param bookContentVO
     * @return
     */
    @PostMapping("/chapterList")
    public Result<List<Chapter>> getChaptersByPageAndRow(@RequestBody BookContentVO bookContentVO) {

        List<Chapter> chapterList = bookService.getChaptersByBookId(bookContentVO.getBookId());
        if (CollectionUtils.isEmpty(chapterList)) {
            return Result.failure(ResultCode.FAIL.getCode(), "哎呀~没有数据啦~");
        } else {
            return Result.byObject(chapterList);
        }
    }

    /**
     * 获取章节书籍内容
     *
     * @param bookContentVO 书籍id,章节id
     * @return
     */
    @PostMapping("/content")
    public Result<BookContent> getBookContentByChapter(@RequestBody BookContentVO bookContentVO) {
        if (bookContentVO.getUserId() == null) {
            return Result.failure(ResultCode.USER_NULL);
        }
        return bookService.getContentByContentId(bookContentVO);
    }

    /**
     * 获取热门搜索
     *
     * @param sex
     * @return
     */
    @GetMapping("/getSearchKeyword")
    public Result<List<SearchKeyword>> getSearchKeywordBySex(String sex) {
        return Result.success(bookService.getSearchKeywordBySex(sex));
    }

    /**
     * 获取指定书籍的阅读记录
     *
     * @param userRecentlyRead
     * @return
     */
    @PostMapping("/getRecentReadByUserIdAndBookId")
    public Result<UserRecentlyRead> getRecentReadByUserIdAndBookId(@RequestBody UserRecentlyRead userRecentlyRead) {
        return Result.success(bookService.getRecentReadByUserIdAndBookId(userRecentlyRead));
    }

    /**
     * 获取指定用户的阅读记录
     *
     * @param userRecentlyRead
     * @return
     */
    @PostMapping("/getRecentReadByUserId")
    public Result<List<UserRecentlyReadRes>> getRecentReadByUserId(@RequestBody UserRecentReadVO userRecentlyRead) {
        return Result.success(bookService.getRecentReadByUserId(userRecentlyRead));
    }

    /**
     * 获取购买章节列表
     *
     * @param bookContentVO
     * @return
     */
    @PostMapping("/getUserConsumerChapter")
    public Result<List<Long>> getUserConsumerChapter(@RequestBody BookContentVO bookContentVO) {
        return Result.success(bookService.getUserConsumerChapter(bookContentVO));
    }

    @Autowired
    private CacheService cacheService;

    private Boolean checkLogin(HttpSession session) {
        Object login = session.getAttribute("login");
        if (login != null && (Boolean) login) {
            return true;
        }
        return false;
    }

    @Autowired
    private BookMapper bookMapper;

    /**
     * 自定义书籍上传
     *
     * @param file
     * @param bookId
     * @return
     * @throws IOException
     */
    @PostMapping("/upload-8F80F0A4-E084-44C6-8659-61612C9D9D44")
    public Result uploadFile(@RequestParam("file") MultipartFile file, Long bookId, HttpSession session) throws IOException {

        if (file.isEmpty() || bookId == null || !cacheService.isUploadBook(bookId)) {
            return Result.failure(ResultCode.PARAMERROR);
        }
        //判断 ID 是否存在
        Book bookById = bookMapper.selectByPrimaryKey(bookId);
        if (bookById != null) {
            return Result.failure(ResultCode.IDEXIST);
        }
        bookService.addFile(file, bookId);
        return Result.success("添加成功");
    }

    /**
     * 后台-删除上传的书籍
     *
     * @param id
     * @return
     */
    @GetMapping("deleteUpload-8F80F0A4-E084-44C6-8659-61612C9D9D44")
    public Result deleteUploadFile(Long id, HttpSession session) {

        return Result.success(bookService.deleteUploadFile(id));
    }

    /**
     * 后台-获取章节书籍内容
     *
     * @param bookContentVO 书籍id,章节id
     * @return
     */
    @PostMapping("/content-05863069-1AE0-435C-9283-2515A4A31BF7")
    public Result<BookContent> getBookContentByChapter2(@RequestBody BookContentVO bookContentVO, HttpSession session) {

        return bookService.getContentByContentId2(bookContentVO);
    }

    /**
     * 后台-更新章节内容
     *
     * @param bookContentVO
     * @return
     */
    @PostMapping("/updateContent-584CBBC1-DBD9-47A9-AAF1-EBA684B82B55")
    public Result updateBookContent(@RequestBody BookContentVO bookContentVO, HttpSession session) {

        return Result.byObject(bookService.updateBookContent(bookContentVO));
    }

    /**
     * 后台-获取建议的 ID
     *
     * @return
     */
    @GetMapping("/getRecommendId-584CBBC1-DBD9-47A9-AAF1-EBC684B82B55")
    public Result getRecommendId(HttpSession session) {
        return Result.byObject(bookService.getRecommendId());
    }

    @Autowired
    private BlockCacheGuava blockCacheGuava;

    /**
     * 后台-更新缓存
     *
     * @return
     */
    @GetMapping("/updateBlockCache-8AA34230-157C-4EF6-98CE-0D928B3DE9EF")
    public Result updateBlockCache(HttpSession session) {
        blockCacheGuava.clearCache();
        return Result.success();
    }

    /**
     * 获取强制关注章节NUM
     *
     * @param appId
     * @param bookId
     * @return
     */
    @GetMapping("/getForceChapterId")
    public Result getForceChapterId(String appId, Integer bookId) {
        return Result.success(cacheService.getBook_guide_chapter_idx(appId, bookId));
    }


    /**
     * 获取关注二维码
     *
     * @param appId
     * @return
     */
    @GetMapping("/getQrcodeUrl")
    public Result getQrcodeUrl(String appId) {
        return Result.byObject(cacheService.getQrcodeUrl(appId));
    }
}
